@@ -273,22 +273,26 @@ List shrinkTVP_cpp(arma::vec y,
   }
 
   // Objects required for stochvol to work
-  arma::mat mixprob(10, N);
-  arma::vec mixprob_vec(mixprob.begin(), mixprob.n_elem, false);
-  arma::ivec r(N);
+  arma::uvec r(N); r.fill(5);
   double h0_samp = as<double>(starting_vals["h0_st"]);
-  double B011inv         = 1e-8;
-  double B022inv         = 1e-12;
-  bool Gammaprior        = true;
-  double MHcontrol       = -1;
-  int parameterization   = 3;
-  bool centered_baseline = parameterization % 2; // 1 for C, 0 for NC baseline
-  int MHsteps = 2;
-  bool dontupdatemu = 0;
-  double cT = N/2.0;
-  double C0_sv = 1.5*Bsigma_sv;
-  bool truncnormal = false;
-  double priorlatent0 = -1;
+  using stochvol::PriorSpec;
+  const PriorSpec prior_spec = {  // prior specification object for the update_*_sv functions
+    PriorSpec::Latent0(),  // stationary prior distribution on priorlatent0
+    PriorSpec::Mu(PriorSpec::Normal(bmu, std::sqrt(Bmu))),  // normal prior on mu
+    PriorSpec::Phi(PriorSpec::Beta(a0_sv, b0_sv)),  // stretched beta prior on phi
+    PriorSpec::Sigma2(PriorSpec::Gamma(0.5, 0.5 / Bsigma_sv))  // normal(0, Bsigma) prior on sigma
+  };  // heavy-tailed, leverage, regression turned off
+  using stochvol::ExpertSpec_FastSV;
+  const ExpertSpec_FastSV expert {  // very expert settings for the Kastner, Fruehwirth-Schnatter (2014) sampler
+    true,  // interweave
+    stochvol::Parameterization::CENTERED,  // centered baseline always
+    1e-8,  // B011inv,
+    1e-12,  //B022inv,
+    2,  // MHsteps,
+    ExpertSpec_FastSV::ProposalSigma2::INDEPENDENCE,  // independece proposal for sigma
+    -1,  // unused for independence prior for sigma
+    ExpertSpec_FastSV::ProposalPhi::IMMEDIATE_ACCEPT_REJECT_NORMAL  // immediately reject (mu,phi,sigma) if proposed phi is outside (-1, 1)
+  };
 
   // Objects necessary for the use of adaptive MH
   arma::mat batches;
@@ -493,38 +497,18 @@ List shrinkTVP_cpp(arma::vec y,
         std::for_each(datastand.begin(), datastand.end(), res_protector);
 
         // update_sv needs sigma and not sigma^2
-        sv_para(2) = std::sqrt(sv_para(2));
+        double mu = sv_para(0);
+        double phi = sv_para(1);
+        double sigma = std::sqrt(sv_para(2));
 
         arma::vec cur_h = arma::log(sigma2_samp);
-        stochvol::update_sv(datastand,
-                            sv_para,
-                            cur_h,
-                            h0_samp,
-                            mixprob_vec,
-                            r,
-                            centered_baseline,
-                            C0_sv,
-                            cT,
-                            Bsigma_sv,
-                            a0_sv,
-                            b0_sv,
-                            bmu,
-                            Bmu,
-                            B011inv,
-                            B022inv,
-                            Gammaprior,
-                            truncnormal,
-                            MHcontrol,
-                            MHsteps,
-                            parameterization,
-                            dontupdatemu,
-                            priorlatent0);
+        stochvol::update_fast_sv(datastand, mu, phi, sigma, h0_samp, cur_h, r, prior_spec, expert);
 
         // Write back into sample object
         sigma2_samp = arma::exp(cur_h);
 
         // change back to sigma^2
-        sv_para(2) = std::pow(sv_para(2), 2);
+        sv_para = {mu, phi, std::pow(sigma, 2)};
 
         std::for_each(sigma2_samp.begin(), sigma2_samp.end(), res_protector);
 
